==> ThbgmExtractor-1.5.0.tar.bz2: ThbgmExtractor-1.5.0/gui/ThbgmExtractor/Smdn.Applications.ThbgmExtractor/Config.cs <==
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;

using Smdn.Collections;
using Smdn.Formats;
using Smdn.Formats.Ini;
using Smdn.Formats.Playlists;
using Smdn.Formats.Thbgm;
using Smdn.Windows.Forms.ThbgmPlayer;

namespace Smdn.Applications.ThbgmExtractor {
  public static class Config {
    internal const string DefaultConfigFileName = "ThbgmExtractor.ini";

    public static class RecentFiles {
      private static string SectionName = "RecentFiles";
      private const int MaxEntries = 10;

      private static List<string> ExtractionConfigFiles = new List<string>();

      public static void ClearExtractionConfigFiles()
      {
        lock (((System.Collections.ICollection)ExtractionConfigFiles).SyncRoot) {
          ExtractionConfigFiles.Clear();
        }
      }

      public static void AddExtractionConfigFile(string file)
      {
        AddExtractionConfigFile(file, true);
      }

      private static void AddExtractionConfigFile(string file, bool insert)
      {
        lock (((System.Collections.ICollection)ExtractionConfigFiles).SyncRoot) {
          if (ExtractionConfigFiles.Contains(file))
            ExtractionConfigFiles.Remove(file);
          else if (MaxEntries <= ExtractionConfigFiles.Count)
            ExtractionConfigFiles.RemoveAt(ExtractionConfigFiles.Count - 1);

          if (insert)
            ExtractionConfigFiles.Insert(0, file);
          else
            ExtractionConfigFiles.Add(file);
        }
      }

      public static string[] GetExtractionConfigFiles()
      {
        lock (((System.Collections.ICollection)ExtractionConfigFiles).SyncRoot) {
          return ExtractionConfigFiles.ToArray();
        }
      }

      internal static void LoadFrom(IniDocument document)
      {
        var section = document[SectionName];
        var entries = new List<KeyValuePair<string, string>>(section.Entries);

        entries.Sort(delegate(KeyValuePair<string, string> x, KeyValuePair<string, string> y) {
          return string.Compare(x.Key, y.Key);
        });

        foreach (var entry in entries) {
          if (entry.Key.StartsWith("ExtractionConfigFile")) {
            if (File.Exists(entry.Value))
              AddExtractionConfigFile(entry.Value, false);
          }
        }
      }

      internal static void SaveTo(IniDocument document)
      {
        var section = document[SectionName];

        var extractionConfigFiles = GetExtractionConfigFiles();

        for (var i = 0; i < extractionConfigFiles.Length; i++) {
          section[string.Format("ExtractionConfigFile{0}", i)] = extractionConfigFiles[i];
        }
      }
    }

    public static class ExtractionConfigDefaults {
      private static string SectionName = "ExtractionConfigDefaults";

      public static bool SaveSourceAfterEncode = false;
      public static PlaylistFormat PlaylistFormat = null;

      internal static void LoadFrom(IniDocument document)
      {
        var section = document[SectionName];

        SaveSourceAfterEncode = section.Get<bool>("SaveSourceAfterEncode", SaveSourceAfterEncode, Convert.ToBoolean);

        var playlistFormatName = section["PlaylistFormat"];

        PlaylistFormat = PlaylistFormat.Formats.FirstOrDefault(delegate(PlaylistFormat format) {
          return format.Name == playlistFormatName;
        }) ?? PlaylistFormat;
      }

      internal static void SaveTo(IniDocument document)
      {
        var section = document[SectionName];

        section["SaveSourceAfterEncode"] = SaveSourceAfterEncode.ToString();
        section["PlaylistFormat"] = (PlaylistFormat == null) ? string.Empty : PlaylistFormat.Name;
      }
    }

    public static class CollectiveEffectEditDefaults {
      private static string SectionName = "CollectiveEffectEditDefaults";

      public static EffectLengthBasis EffectLengthBasis = EffectLengthBasis.FadingStart;
      public static FadingCurveType FadingCurveType = FadingCurveType.Linear;
      public static double FadingCurveCoefficient = 1.0;
      public static int TimesToRepeat = 2; // 2 loops
      public static TimeSpan FadingStart = TimeSpan.FromMinutes(5.0); // 5 mins
      public static TimeSpan FadingOffset = TimeSpan.Zero; // 0 secs
      public static TimeSpan FadingLength = TimeSpan.FromSeconds(20.0); // 20 secs
      public static TimeSpan TotalLength = TimeSpan.FromMinutes(5.0); // 5 mins
      public static TimeSpan TrailingSilenceLength = TimeSpan.Zero; // 0 secs

      internal static void LoadFrom(IniDocument document)
      {
        var section = document[SectionName];

        EffectLengthBasis       = section.Get<EffectLengthBasis>("EffectLengthBasis", EffectLengthBasis, EnumUtils.Parse<EffectLengthBasis>);
        FadingCurveType         = section.Get<FadingCurveType>("FadingCurveType", FadingCurveType, EnumUtils.Parse<FadingCurveType>);
        FadingCurveCoefficient  = section.Get<double>("FadingCurveCoefficient", FadingCurveCoefficient, Convert.ToDouble);
        TimesToRepeat           = section.Get<int>("TimesToRepeat", TimesToRepeat, Convert.ToInt32);
        FadingStart             = TimeSpanUtils.ParsePositive(section["FadingStart"], FadingStart);
        FadingOffset            = TimeSpanUtils.Parse(section["FadingOffset"], FadingOffset);
        FadingLength            = TimeSpanUtils.ParsePositive(section["FadingLength"], FadingLength);
        TotalLength             = TimeSpanUtils.ParsePositive(section["TotalLength"], TotalLength);
        TrailingSilenceLength   = TimeSpanUtils.ParsePositive(section["TrailingSilenceLength"], TrailingSilenceLength);

        if (double.IsNaN(FadingCurveCoefficient) || double.IsInfinity(FadingCurveCoefficient))
          FadingCurveCoefficient = 1.0;

        if (TimesToRepeat < 0)
          TimesToRepeat = 0;
      }

      internal static void SaveTo(IniDocument document)
      {
        var section = document[SectionName];

        section["EffectLengthBasis"]      = EffectLengthBasis.ToString();
        section["FadingCurveType"]        = FadingCurveType.ToString();
        section["FadingCurveCoefficient"] = FadingCurveCoefficient.ToString();
        section["TimesToRepeat"]          = TimesToRepeat.ToString();
        section["FadingStart"]            = FadingStart.ToString();
        section["FadingOffset"]           = FadingOffset.ToString();
        section["FadingLength"]           = FadingLength.ToString();
        section["TotalLength"]            = TotalLength.ToString();
        section["TrailingSilenceLength"]  = TrailingSilenceLength.ToString();
      }

      internal static void OpenConfigAppliedDialog(Action<CollectiveEffectEditDialog> action)
      {
        using (var dialog = new CollectiveEffectEditDialog()) {
          Apply(dialog);

          action(dialog);

          if (dialog.DialogResult == System.Windows.Forms.DialogResult.OK)
            Store(dialog);
        }
      }

      internal static void Apply(CollectiveEffectEditDialog dialog)
      {
        dialog.EffectLengthBasis      = EffectLengthBasis;
        dialog.TimesToRepeat          = TimesToRepeat;
        dialog.FadingStart            = FadingStart;
        dialog.FadingOffset           = FadingOffset;
        dialog.FadingLength           = FadingLength;
        dialog.TotalLength            = TotalLength;
        dialog.TrailingSilenceLength  = TrailingSilenceLength;

        dialog.SetFadingCurveValues(FadingCurveType, FadingCurveCoefficient);
      }

      internal static void Store(CollectiveEffectEditDialog dialog)
      {
        EffectLengthBasis      = dialog.EffectLengthBasis;
        FadingCurveType        = dialog.FadingCurveType;
        FadingCurveCoefficient = dialog.FadingCurveCoefficient;
        TimesToRepeat          = dialog.TimesToRepeat;
        FadingStart            = dialog.FadingStart;
        FadingOffset           = dialog.FadingOffset;
        FadingLength           = dialog.FadingLength;
        TotalLength            = dialog.TotalLength;
        TrailingSilenceLength  = dialog.TrailingSilenceLength;
      }
    }

    public static class TrackFormatInfoDefaults {
      private static string SectionName = "TrackFormatInfoDefaults";

      public static string DefaultOutputFileName = "$trackno. $title";
      public static TrackFormatInfo TrackFormatInfo;

      static TrackFormatInfoDefaults()
      {
        TrackFormatInfo = new TrackFormatInfo();
        TrackFormatInfo.ZeroPadding = true;
        TrackFormatInfo.TitleUnificationOptions = TitleUnificationOptions.FullWidthTilde | TitleUnificationOptions.SingleIdeographicSpace;
      }

      internal static void LoadFrom(IniDocument document)
      {
        var section = document[SectionName];

        DefaultOutputFileName = section.Get("DefaultOutputFileName", DefaultOutputFileName);
        TrackFormatInfo.TitleUnificationOptions = section.Get<TitleUnificationOptions>("TitleUnificationOptions", TrackFormatInfo.TitleUnificationOptions, EnumUtils.Parse<TitleUnificationOptions>);
        TrackFormatInfo.ZeroPadding = section.Get<bool>("ZeroPadding", TrackFormatInfo.ZeroPadding, Convert.ToBoolean);
      }

      internal static void SaveTo(IniDocument document)
      {
        var section = document[SectionName];

        section["DefaultOutputFileName"] = DefaultOutputFileName;
        section["TitleUnificationOptions"] = TrackFormatInfo.TitleUnificationOptions.ToString();
        section["ZeroPadding"] = TrackFormatInfo.ZeroPadding.ToString();
      }

      internal static void OpenConfigAppliedCollectiveTrackTagEditDialog(Action<CollectiveTrackTagEditDialog> action)
      {
        using (var dialog = new CollectiveTrackTagEditDialog()) {
          Apply(dialog);

          action(dialog);

          if (dialog.DialogResult == System.Windows.Forms.DialogResult.OK)
            Store(dialog);
        }
      }

      internal static void Apply(CollectiveTrackTagEditDialog dialog)
      {
        dialog.OutputFileName = DefaultOutputFileName;
        dialog.TrackFormatInfo = TrackFormatInfo.Clone();
      }

      internal static void Store(CollectiveTrackTagEditDialog dialog)
      {
        DefaultOutputFileName = dialog.OutputFileName;

        if (dialog.TrackFormatInfo == null)
          TrackFormatInfo = new TrackFormatInfo();
        else
          TrackFormatInfo = dialog.TrackFormatInfo.Clone();
      }
    }

    public static class UserEnvironmentProductInfoes {
      private const string sectionPrefix = "ProductInfo:";

      private static string GetSectionNameFrom(ProductInfo product)
      {
        if (string.IsNullOrEmpty(product.Title))
          // can't store, ignore
          return null;
        else
          return sectionPrefix + product.Title;
      }

      private static string GetProductTitleFrom(string sectionName)
      {
        if (sectionName.StartsWith(sectionPrefix))
          return sectionName.Substring(sectionPrefix.Length);
        else
          return null;
      }

      internal static void LoadFrom(IniDocument document, IEnumerable<ProductInfo> products)
      {
        var productList = new List<ProductInfo>(products);

        foreach (var section in document) {
          var title = GetProductTitleFrom(section.Name);
          var product = productList.Find(delegate(ProductInfo p) {
            return p.Title == title;
          });

          if (product == null)
            continue;

          var installedBgmSourcePath = section["InstalledBgmSourcePath"];

          if (File.Exists(installedBgmSourcePath))
            product.InstalledBgmSourcePath = installedBgmSourcePath;
        }
      }

      internal static void SaveTo(IniDocument document, IEnumerable<ProductInfo> products)
      {
        foreach (var product in products) {
          var sectionName = GetSectionNameFrom(product);

          if (sectionName == null)
            continue;

          var section = document[sectionName];

          if (!string.IsNullOrEmpty(product.InstalledBgmSourcePath))
            section["InstalledBgmSourcePath"] = product.InstalledBgmSourcePath;

          // delete empty section
          if (section.Entries.Count == 0)
            document.Remove(section.Name);
        }
      }
    }

    public static void Load(string searchDirectory, IEnumerable<ProductInfo> products)
    {
      configFilePath = Path.Combine(searchDirectory, DefaultConfigFileName);

      if (!File.Exists(configFilePath)) {
        Console.WriteLine("configuration file '{0}' not found", configFilePath);
        return;
      }

      try {
        var document = IniDocument.Load(configFilePath, StringComparer.CurrentCulture);

        RecentFiles.LoadFrom(document);
        ExtractionConfigDefaults.LoadFrom(document);
        CollectiveEffectEditDefaults.LoadFrom(document);
        TrackFormatInfoDefaults.LoadFrom(document);
        UserEnvironmentProductInfoes.LoadFrom(document, products);
      }
      catch {
        // ignore any exceptions
      }
    }

    public static void Save(IEnumerable<ProductInfo> products)
    {
      try {
        var document = new IniDocument(StringComparer.CurrentCulture);

        RecentFiles.SaveTo(document);
        ExtractionConfigDefaults.SaveTo(document);
        CollectiveEffectEditDefaults.SaveTo(document);
        TrackFormatInfoDefaults.SaveTo(document);
        UserEnvironmentProductInfoes.SaveTo(document, products);

        document.Save(configFilePath);
      }
      catch {
        // ignore any exceptions
      }
    }

    private static string configFilePath;
  }
}

==> ThbgmExtractor-1.6.0.tar.bz2: ThbgmExtractor-1.6.0/gui/ThbgmExtractor/Smdn.Applications.ThbgmExtractor/Config.cs <==
//  
//  Author:
//       smdn <smdn@smdn.jp>
// 
//  Copyright (c) 2004-2013 smdn
// 
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Windows.Forms;

using Smdn.Collections;
using Smdn.Formats;
using Smdn.Formats.Ini;
using Smdn.Formats.Playlists;
using Smdn.Formats.Thbgm;
using Smdn.Windows.Forms.ThbgmPlayer;

namespace Smdn.Applications.ThbgmExtractor {
  public static class Config {
    internal const string DefaultConfigFileName = "ThbgmExtractor.ini";

    public static class RecentFiles {
      private const string SectionName = "RecentFiles";
      private const int MaxEntries = 10;

      private static List<string> ExtractionConfigFiles = new List<string>();

      public static void ClearExtractionConfigFiles()
      {
        lock (((System.Collections.ICollection)ExtractionConfigFiles).SyncRoot) {
          ExtractionConfigFiles.Clear();
        }
      }

      public static void AddExtractionConfigFile(string file)
      {
        AddExtractionConfigFile(file, true);
      }

      private static void AddExtractionConfigFile(string file, bool insert)
      {
        lock (((System.Collections.ICollection)ExtractionConfigFiles).SyncRoot) {
          if (ExtractionConfigFiles.Contains(file))
            ExtractionConfigFiles.Remove(file);
          else if (MaxEntries <= ExtractionConfigFiles.Count)
            ExtractionConfigFiles.RemoveAt(ExtractionConfigFiles.Count - 1);

          if (insert)
            ExtractionConfigFiles.Insert(0, file);
          else
            ExtractionConfigFiles.Add(file);
        }
      }

      public static string[] GetExtractionConfigFiles()
      {
        lock (((System.Collections.ICollection)ExtractionConfigFiles).SyncRoot) {
          return ExtractionConfigFiles.ToArray();
        }
      }

      internal static void LoadFrom(IniDocument document)
      {
        var section = document[SectionName];
        var entries = new List<KeyValuePair<string, string>>(section.Entries);

        entries.Sort(delegate(KeyValuePair<string, string> x, KeyValuePair<string, string> y) {
          return string.Compare(x.Key, y.Key);
        });

        foreach (var entry in entries) {
          if (entry.Key.StartsWith("ExtractionConfigFile", StringComparison.Ordinal)) {
            if (File.Exists(entry.Value))
              AddExtractionConfigFile(entry.Value, false);
          }
        }
      }

      internal static void SaveTo(IniDocument document)
      {
        var section = document[SectionName];

        var extractionConfigFiles = GetExtractionConfigFiles();

        for (var i = 0; i < extractionConfigFiles.Length; i++) {
          section[string.Format("ExtractionConfigFile{0}", i)] = extractionConfigFiles[i];
        }
      }
    }

    public static class MainDialogState {
      private const string SectionName = "MainDialogStates";
      private static readonly Size MinimumSize = new Size(640, 320); // XXX

      public static Rectangle Bounds = Rectangle.Empty;
      public static FormWindowState WindowState = FormWindowState.Normal;

      public static void Store(MainDialog dialog)
      {
        if (dialog.WindowState == FormWindowState.Normal)
          Bounds = dialog.Bounds;
        else
          Bounds = dialog.RestoreBounds;

        WindowState = dialog.WindowState;
      }

      public static void Restore(MainDialog dialog)
      {
        if (MinimumSize.Width <= Bounds.Width &&
            MinimumSize.Height <= Bounds.Height)
          dialog.Bounds = Bounds;

        if (WindowState != FormWindowState.Minimized)
          dialog.WindowState = WindowState;
      }

      internal static void LoadFrom(IniDocument document)
      {
        var section = document[SectionName];

        var left    = section.Get("Bounds.Left", Rectangle.Empty.Left, int.Parse);
        var top     = section.Get("Bounds.Top", Rectangle.Empty.Top, int.Parse);
        var width   = section.Get("Bounds.Width", Rectangle.Empty.Width, int.Parse);
        var height  = section.Get("Bounds.Height", Rectangle.Empty.Height, int.Parse);

        Bounds = new Rectangle(left, top, width, height);

        WindowState = section.Get("WindowState", FormWindowState.Normal, EnumUtils.Parse<FormWindowState>);
      }

      internal static void SaveTo(IniDocument document)
      {
        var section = document[SectionName];

        section["Bounds.Left"]    = Bounds.Left.ToString();
        section["Bounds.Top"]     = Bounds.Top.ToString();
        section["Bounds.Width"]   = Bounds.Width.ToString();
        section["Bounds.Height"]  = Bounds.Height.ToString();
        section["WindowState"]    = WindowState.ToString();
      }
    }

    public static class ExtractionConfigDefaults {
      private const string SectionName = "ExtractionConfigDefaults";

      public static bool SaveSourceAfterEncode = false;
      public static PlaylistFormat PlaylistFormat = null;
      public static bool EnableTagging = true;

      internal static void LoadFrom(IniDocument document)
      {
        var section = document[SectionName];

        SaveSourceAfterEncode = section.Get<bool>("SaveSourceAfterEncode", SaveSourceAfterEncode, Convert.ToBoolean);

        var playlistFormatName = section["PlaylistFormat"];

        PlaylistFormat = PlaylistFormat.Formats.FirstOrDefault(delegate(PlaylistFormat format) {
          return format.Name == playlistFormatName;
        }) ?? PlaylistFormat;

        EnableTagging = section.Get<bool>("EnableTagging", EnableTagging, Convert.ToBoolean);
      }

      internal static void SaveTo(IniDocument document)
      {
        var section = document[SectionName];

        section["SaveSourceAfterEncode"] = SaveSourceAfterEncode.ToString();
        section["PlaylistFormat"] = (PlaylistFormat == null) ? string.Empty : PlaylistFormat.Name;
        section["EnableTagging"] = EnableTagging.ToString();
      }
    }

    public static class CollectiveEffectEditDefaults {
      private const string SectionName = "CollectiveEffectEditDefaults";

      public static EffectLengthBasis EffectLengthBasis = EffectLengthBasis.FadingStart;
      public static FadingCurveType FadingCurveType = FadingCurveType.Linear;
      public static double FadingCurveCoefficient = 1.0;
      public static int TimesToRepeat = 2; // 2 loops
      public static TimeSpan FadingStart = TimeSpan.FromMinutes(5.0); // 5 mins
      public static TimeSpan FadingOffset = TimeSpan.Zero; // 0 secs
      public static TimeSpan FadingLength = TimeSpan.FromSeconds(20.0); // 20 secs
      public static TimeSpan TotalLength = TimeSpan.FromMinutes(5.0); // 5 mins
      public static TimeSpan TrailingSilenceLength = TimeSpan.Zero; // 0 secs

      internal static void LoadFrom(IniDocument document)
      {
        var section = document[SectionName];

        EffectLengthBasis       = section.Get<EffectLengthBasis>("EffectLengthBasis", EffectLengthBasis, EnumUtils.Parse<EffectLengthBasis>);
        FadingCurveType         = section.Get<FadingCurveType>("FadingCurveType", FadingCurveType, EnumUtils.Parse<FadingCurveType>);
        FadingCurveCoefficient  = section.Get<double>("FadingCurveCoefficient", FadingCurveCoefficient, Convert.ToDouble);
        TimesToRepeat           = section.Get<int>("TimesToRepeat", TimesToRepeat, Convert.ToInt32);
        FadingStart             = TimeSpanUtils.ParsePositive(section["FadingStart"], FadingStart);
        FadingOffset            = TimeSpanUtils.Parse(section["FadingOffset"], FadingOffset);
        FadingLength            = TimeSpanUtils.ParsePositive(section["FadingLength"], FadingLength);
        TotalLength             = TimeSpanUtils.ParsePositive(section["TotalLength"], TotalLength);
        TrailingSilenceLength   = TimeSpanUtils.ParsePositive(section["TrailingSilenceLength"], TrailingSilenceLength);

        if (double.IsNaN(FadingCurveCoefficient) || double.IsInfinity(FadingCurveCoefficient))
          FadingCurveCoefficient = 1.0;

        if (TimesToRepeat < 0)
          TimesToRepeat = 0;
      }

      internal static void SaveTo(IniDocument document)
      {
        var section = document[SectionName];

        section["EffectLengthBasis"]      = EffectLengthBasis.ToString();
        section["FadingCurveType"]        = FadingCurveType.ToString();
        section["FadingCurveCoefficient"] = FadingCurveCoefficient.ToString();
        section["TimesToRepeat"]          = TimesToRepeat.ToString();
        section["FadingStart"]            = FadingStart.ToString();
        section["FadingOffset"]           = FadingOffset.ToString();
        section["FadingLength"]           = FadingLength.ToString();
        section["TotalLength"]            = TotalLength.ToString();
        section["TrailingSilenceLength"]  = TrailingSilenceLength.ToString();
      }

      internal static void OpenConfigAppliedDialog(Action<CollectiveEffectEditDialog> action)
      {
        using (var dialog = new CollectiveEffectEditDialog()) {
          Apply(dialog);

          action(dialog);

          if (dialog.DialogResult == System.Windows.Forms.DialogResult.OK)
            Store(dialog);
        }
      }

      internal static void Apply(CollectiveEffectEditDialog dialog)
      {
        dialog.EffectLengthBasis      = EffectLengthBasis;
        dialog.TimesToRepeat          = TimesToRepeat;
        dialog.FadingStart            = FadingStart;
        dialog.FadingOffset           = FadingOffset;
        dialog.FadingLength           = FadingLength;
        dialog.TotalLength            = TotalLength;
        dialog.TrailingSilenceLength  = TrailingSilenceLength;

        dialog.SetFadingCurveValues(FadingCurveType, FadingCurveCoefficient);
      }

      internal static void Store(CollectiveEffectEditDialog dialog)
      {
        EffectLengthBasis      = dialog.EffectLengthBasis;
        FadingCurveType        = dialog.FadingCurveType;
        FadingCurveCoefficient = dialog.FadingCurveCoefficient;
        TimesToRepeat          = dialog.TimesToRepeat;
        FadingStart            = dialog.FadingStart;
        FadingOffset           = dialog.FadingOffset;
        FadingLength           = dialog.FadingLength;
        TotalLength            = dialog.TotalLength;
        TrailingSilenceLength  = dialog.TrailingSilenceLength;
      }
    }

    public static class TrackFormatInfoDefaults {
      private const string SectionName = "TrackFormatInfoDefaults";

      public static string DefaultOutputFileName = "$trackno. $title";
      public static TrackFormatInfo TrackFormatInfo;

      static TrackFormatInfoDefaults()
      {
        TrackFormatInfo = new TrackFormatInfo();
        TrackFormatInfo.ZeroPadding = true;
        TrackFormatInfo.TitleUnificationOptions = TitleUnificationOptions.FullWidthTilde | TitleUnificationOptions.SingleIdeographicSpace;
      }

      internal static void LoadFrom(IniDocument document)
      {
        var section = document[SectionName];

        DefaultOutputFileName = section.Get("DefaultOutputFileName", DefaultOutputFileName);
        TrackFormatInfo.TitleUnificationOptions = section.Get<TitleUnificationOptions>("TitleUnificationOptions", TrackFormatInfo.TitleUnificationOptions, EnumUtils.Parse<TitleUnificationOptions>);
        TrackFormatInfo.ZeroPadding = section.Get<bool>("ZeroPadding", TrackFormatInfo.ZeroPadding, Convert.ToBoolean);
      }

      internal static void SaveTo(IniDocument document)
      {
        var section = document[SectionName];

        section["DefaultOutputFileName"] = DefaultOutputFileName;
        section["TitleUnificationOptions"] = TrackFormatInfo.TitleUnificationOptions.ToString();
        section["ZeroPadding"] = TrackFormatInfo.ZeroPadding.ToString();
      }

      internal static void OpenConfigAppliedCollectiveTrackTagEditDialog(Action<CollectiveTrackTagEditDialog> action)
      {
        using (var dialog = new CollectiveTrackTagEditDialog()) {
          Apply(dialog);

          action(dialog);

          if (dialog.DialogResult == System.Windows.Forms.DialogResult.OK)
            Store(dialog);
        }
      }

      internal static void Apply(CollectiveTrackTagEditDialog dialog)
      {
        dialog.OutputFileName = DefaultOutputFileName;
        dialog.TrackFormatInfo = TrackFormatInfo.Clone();
      }

      internal static void Store(CollectiveTrackTagEditDialog dialog)
      {
        DefaultOutputFileName = dialog.OutputFileName;

        if (dialog.TrackFormatInfo == null)
          TrackFormatInfo = new TrackFormatInfo();
        else
          TrackFormatInfo = dialog.TrackFormatInfo.Clone();
      }
    }

    public static class UserEnvironmentProductInfoes {
      private const string sectionPrefix = "ProductInfo:";

      private static string GetSectionNameFrom(ProductInfo product)
      {
        if (string.IsNullOrEmpty(product.Title))
          // can't store, ignore
          return null;
        else
          return sectionPrefix + product.Title;
      }

      private static string GetProductTitleFrom(string sectionName)
      {
        if (sectionName.StartsWith(sectionPrefix, StringComparison.Ordinal))
          return sectionName.Substring(sectionPrefix.Length);
        else
          return null;
      }

      internal static void LoadFrom(IniDocument document, ReadOnlyCollection<ProductInfo> products)
      {
        foreach (var section in document) {
          var title = GetProductTitleFrom(section.Name);
          var product = products.Where(delegate(ProductInfo p) {
            return p.Title == title;
          }).FirstOrDefault();

          if (product == null)
            continue;

          var installedBgmSourcePath = section["InstalledBgmSourcePath"];

          if (File.Exists(installedBgmSourcePath))
            product.InstalledBgmSourcePath = installedBgmSourcePath;
        }
      }

      internal static void SaveTo(IniDocument document, ReadOnlyCollection<ProductInfo> products)
      {
        foreach (var product in products) {
          var sectionName = GetSectionNameFrom(product);

          if (sectionName == null)
            continue;

          var section = document[sectionName];

          if (!string.IsNullOrEmpty(product.InstalledBgmSourcePath))
            section["InstalledBgmSourcePath"] = product.InstalledBgmSourcePath;

          // delete empty section
          if (section.Entries.Count == 0)
            document.Remove(section.Name);
        }
      }
    }

    private static void Open(string searchDirectory, Action<IniDocument> action)
    {
      configFilePath = Path.Combine(searchDirectory, DefaultConfigFileName);

      if (!File.Exists(configFilePath)) {
        Console.WriteLine("configuration file '{0}' not found", configFilePath);
        return;
      }

      try {
        action(IniDocument.Load(configFilePath, StringComparer.CurrentCulture));
      }
      catch (Exception ex) {
        Console.WriteLine("error in configuration file '{0}'", configFilePath);
        Console.WriteLine(ex);
      }
    }

    public static void Load(string searchDirectory)
    {
      Open(searchDirectory, delegate(IniDocument document) {
        RecentFiles.LoadFrom(document);
        MainDialogState.LoadFrom(document);
        ExtractionConfigDefaults.LoadFrom(document);
        CollectiveEffectEditDefaults.LoadFrom(document);
        TrackFormatInfoDefaults.LoadFrom(document);
      });
    }

    public static void Load(string searchDirectory, ReadOnlyCollection<ProductInfo> products)
    {
      Open(searchDirectory, delegate(IniDocument document) {
        UserEnvironmentProductInfoes.LoadFrom(document, products);
      });
    }

    public static void Save(ReadOnlyCollection<ProductInfo> products)
    {
      try {
        var document = new IniDocument(StringComparer.CurrentCulture);

        RecentFiles.SaveTo(document);
        MainDialogState.SaveTo(document);
        ExtractionConfigDefaults.SaveTo(document);
        CollectiveEffectEditDefaults.SaveTo(document);
        TrackFormatInfoDefaults.SaveTo(document);
        UserEnvironmentProductInfoes.SaveTo(document, products);

        document.Save(configFilePath);
      }
      catch {
        // ignore any exceptions
      }
    }

    private static string configFilePath;
  }
}
